using System;
using System.Collections.Generic;
using System.Text;

namespace Sketcher
{

    class RadialPopUpMenuOnCrossBehavior : ItemInputBehavior
    {
        RadialMenu m_menu;

        bool m_bSelected;

        public RadialPopUpMenuOnCrossBehavior(Item target)
            : base(target) 
        {
            m_menu = new RadialMenu(target, 30);

            // add behavior to target
            m_bSelected = false;
        }

        //! return cross "factor"  (length of cross inside region)
        //override public float GetCrossScore(LineStroke s) {
        //    float fScore = StrokeUtil.GetCrossScore(ITarget.Bounds, s, null);
        //    return fScore;
        //}

        override public FeatureVector GetCrossFeatures(LineStroke s) {
            float fScore = StrokeUtil.GetCrossScore(TargetItem.Bounds, s, null);
            if ( fScore == 0 )
                return null;
            FeatureVector fv = new FeatureVector(0);
            fv.Add(fScore);
            return fv;
        }


        //! handle cross event
        override public void OnCross(LineStroke s) {
            if (! m_bSelected) {
                m_menu.Center = TargetItem.Bounds.Max;
                m_menu.Show();

                m_bSelected = true;
            } else {
                if (m_menu != null)
                    m_menu.Hide();

                m_bSelected = false;
            }
        }

    }



    class ToggleBGColorOnCircleBehavior : ItemInputBehavior
    {
        bool m_bSelected;
        ColorRGBA m_cOriginal;
        //FadeColorBehavior m_selectionAnimation;

        public ToggleBGColorOnCircleBehavior(Item target)
            : base(target) 
        {
            // add behavior to target
            m_bSelected = false;
            m_cOriginal = target.BackgroundColor;
            //m_selectionAnimation = new FadeColorBehavior( target );
            //target.AppendBehavior( m_selectionAnimation );
        }

        //! return circle "factor" 
        //! (should be percentage of circle inside region, but currently just 1/0 depending on containment of center)
        override public float GetCircleScore(CircleStroke s) {
            if (TargetItem.Bounds.Contains(s.Center))
                return 1;
            return 0;
        }

        //! handle circle event
        override public void OnCircle(CircleStroke s) {
            if (! m_bSelected) {
                //AnimationBehaviorManager.Get.StartAnimation( ITarget, m_selectionAnimation );
                TargetItem.BackgroundColor = ColorRGBA.GREEN;
                m_bSelected = true;
            } else {
                //AnimationBehaviorManager.Get.StopAnimation( ITarget, m_selectionAnimation );
                TargetItem.BackgroundColor = m_cOriginal;
                m_bSelected = false;
            }
        }
    }





    class FadeColorBehavior : RenderableAnimationBehavior
    {
        FadeToGrayAnimation m_fadeAnim;
        
        public FadeColorBehavior(Renderable target)
            : base(target) 
        {
        }

        override public void OnStart() {
            m_fadeAnim = new FadeToGrayAnimation(RTarget, float.MaxValue, 3.5f );
            m_fadeAnim.AnimationFinishedEvent += new Animation.AnimationFinishedHandler(OnStopCallback);

            AnimationManager.Get.AnimateOnce( m_fadeAnim );            
        }
        override public void OnStop() {
            if ( m_fadeAnim.Playing )
                AnimationManager.Get.StopAnimation(m_fadeAnim);
            RTarget.BackgroundColor = m_fadeAnim.OriginalColor;
        }
        protected void OnStopCallback(object o) {
            OnStop();
        }
    }
}
