using System;
using System.Collections.Generic;
using System.Text;

namespace Sketcher
{
    /*
     * [TODO] remove container list subset, use List.Find to select containers...
     */

    class ItemSet
    {
        List<Item> m_items;
        List<Container> m_containers;   // subset of items

        public ItemSet() {
            m_items = new List<Item>();
            m_containers = new List<Container>();
        }

        public int Count {
            get { return m_items.Count; }
        }

        public Item this[int i] {
            get { return m_items[i]; }
        }

        public bool Contains(Item item) {
            return m_items.Contains(item);
        }

        public int ContainerCount {
            get { return m_containers.Count; }
        }
        public Container GetContainer(int i) {
            return m_containers[i];
        }


        //! returns -1 if item not found
        public int FindIndex(Item item) {
            // [TODO] replace with something faster than linear search?  ( list.BinarySearch seemed to be broken... )
            for (int i = 0; i < m_items.Count; ++i)
                if (m_items[i] == item)
                    return i;
            return -1;
        }


        public void Add(Item item) {
            m_items.Add(item);
            if ((item as Container) != null)
                m_containers.Add(item as Container);
            ItemSetChangedArgs e = new ItemSetChangedArgs(item, true, false);
            NotifyItemSetChanged(e);
        }
        public void Add(Item item, int nIndex) {
            m_items.Insert(nIndex, item);
            if ((item as Container) != null)
                m_containers.Add(item as Container);
            ItemSetChangedArgs e = new ItemSetChangedArgs(item, true, false);
            NotifyItemSetChanged(e);
        }
        public void Add( List<Item> items ) {
            for ( int i = 0; i < items.Count; ++i ) {
                m_items.Add(items[i]);
            }

            ItemSetChangedArgs e = new ItemSetChangedArgs(items, true, false);
            NotifyItemSetChanged(e);
        }
        public void Add( List<Item> items, int nIndex ) {
            if (items.Count == 0)
                return;

            for (int i = 0; i < items.Count; ++i) {
                m_items.Insert( nIndex, items[i] );
            }

            ItemSetChangedArgs e = new ItemSetChangedArgs( items, true, false );
            NotifyItemSetChanged( e );
        }




        public void Remove(Item item) {
            Container c = item as Container;
            if (c != null)
                m_containers.Remove(c);
            m_items.Remove(item);

            ItemSetChangedArgs e = new ItemSetChangedArgs(item, false, true);
            NotifyItemSetChanged(e);
        }

        public void Remove(List<Item> items) {
            for (int i = 0; i < items.Count; ++i) {
                Container c = items[i] as Container;
                if (c != null)
                    m_containers.Remove(c);
                m_items.Remove(items[i]);
            }
            ItemSetChangedArgs e = new ItemSetChangedArgs(items, false, true);
            NotifyItemSetChanged(e);
        }



        public bool Replace( Item replace, List<Item> vWith ) {
            int nFound = FindIndex(replace);
            if (nFound < 0)
                return false;
            Add( vWith, nFound );
            Remove( replace );
            return true;
        }
        public bool Replace(List<Item> vReplace, Item with) {
            // [TODO] replace with something faster than linear search?  ( list.BinarySearch seemed to be broken... )
            for (int i = 0; i < m_items.Count; ++i) {
                if (!vReplace.Contains(m_items[i]))
                    continue;
                int nFound = i;
                if (nFound >= 0) {
                    Add(with, nFound);
                    Remove(vReplace);
                    return true;
                }
            }
            return false;
        }


        public void Clear() {
            if (m_items.Count == 0)
                return;
            List<Item> all = new List<Item>(m_items);
            m_items.Clear();

            ItemSetChangedArgs e = new ItemSetChangedArgs(all, false, true);
            NotifyItemSetChanged(e);
        }


        public delegate void ItemSetChangedHandler(object o, ItemSetChangedArgs e);
        public event ItemSetChangedHandler ItemSetChangedEvent;

        //! change set event argument
        public class ItemSetChangedArgs : EventArgs
        {
            public readonly List<Item> added;
            public readonly List<Item> removed;

            public ItemSetChangedArgs(Item item, bool bAdded, bool bRemoved) {
                added = removed = null;
                if (bAdded) {
                    added = new List<Item>();
                    added.Add(item);
                }
                if (bRemoved) {
                    removed = new List<Item>();
                    removed.Add(item);
                }
            }

            public ItemSetChangedArgs(List<Item> items, bool bAdded, bool bRemoved) {
                added = removed = null;
                if (bAdded) {
                    added = new List<Item>(items);
                }
                if (bRemoved) {
                    removed = new List<Item>(items);
                }
            }
        }

        //! raise item changed event
        protected void NotifyItemSetChanged(ItemSetChangedArgs e) {
            if (ItemSetChangedEvent != null)
                ItemSetChangedEvent(this, e);
        } 

    }
}
