using System;
using System.Collections.Generic;
using System.Text;

namespace Sketcher {

    class RenderingBehaviorManager
    {
        List<RenderingBehavior> m_targets;
        Dictionary<object, List<RenderingBehavior> > m_targetMap;

        public void Register(RenderingBehavior behavior) {
            if (!m_targets.Contains(behavior)) {
                m_targets.Add(behavior);
                if (!m_targetMap.ContainsKey(behavior.Target))
                    m_targetMap.Add(behavior.Target, new List<RenderingBehavior>());
                m_targetMap[behavior.Target].Add(behavior);
            }
        }
        public void DeRegister(RenderingBehavior behavior) {
            m_targets.Remove(behavior);
            if (m_targetMap.ContainsKey(behavior.Target))
                m_targetMap[behavior.Target].Remove(behavior);
        }




        public bool UpdateOverlayBackground( object filterObject ) {
            if (filterObject != null && !m_targetMap.ContainsKey(filterObject))
                return false;

            bool bUpdate = false;

            List<RenderingBehavior> vList = (filterObject == null) ? m_targets : m_targetMap[filterObject];
            foreach (RenderingBehavior r in vList) {
                if (r.Enabled && r.UpdateOverlayFlag) {
                    bool bUpdatedItem = r.OnUpdateOverlayBackground();
                    bUpdate = bUpdate || bUpdatedItem;
                }
            }
            return bUpdate;
        }



        public bool OverlayRendererRequested(object filterObject) {
            if (filterObject != null && !m_targetMap.ContainsKey(filterObject))
                return false;
            List<RenderingBehavior> vList = (filterObject == null) ? m_targets : m_targetMap[filterObject];
            foreach (RenderingBehavior r in vList) {
                if (r.Enabled && r.OverlayRenderFlag)
                    return true;
            }
            return false;
        }


        public bool OverlayRender(object filterObject) {
            if (filterObject != null && !m_targetMap.ContainsKey(filterObject))
                return false;

            bool bUpdate = false;

            List<RenderingBehavior> vList = (filterObject == null) ? m_targets : m_targetMap[filterObject];
            foreach (RenderingBehavior r in vList) {
                if (r.Enabled && r.OverlayRenderFlag) {
                    bool bUpdatedItem = r.OnOverlayRender();
                    bUpdate = bUpdate || bUpdatedItem;
                }
            }
            return bUpdate;
        }




        static public RenderingBehaviorManager Get {
            get {
                if (s_singleton == null)
                    s_singleton = new RenderingBehaviorManager();
                return s_singleton;
            }
        }

        protected RenderingBehaviorManager() {
            m_targets = new List<RenderingBehavior>();
            m_targetMap = new Dictionary<object, List<RenderingBehavior>>();
        }
        static RenderingBehaviorManager s_singleton = null;
    }
}
