using System;
using System.Collections.Generic;
using System.Text;

namespace Sketcher
{
    class FingerKeyButton : Item
    {
        Box2f m_box;

        Image m_discShadow;
        float m_fDropShadowWidth;

        string m_text;

        public FingerKeyButton(float fWidth, float fHeight, string text) {
            Resize(new Box2f(fWidth, fHeight));
            m_box = Bounds;
            m_text = text;

            m_fDropShadowWidth = 20.0f;

            int nTexSize = 128;
            float fInnerRad = 0.25f;

            // creates shadow box of right dimensions
            m_discShadow = new Image(nTexSize, nTexSize);
            Image.Pixel  pix = new Image.Pixel(0.0f, 0.0f, 0.0f);
            float fRatio = m_box.Width / m_box.Height;
            Box2f shadowBounds = new Box2f( -fRatio, -1, fRatio, 1 );
            Box2f shadowBox = new Box2f(shadowBounds);
            shadowBox.Expand(-(1-fInnerRad));

            // ok now create falloff image 
            // TODO: kind of broken - scaling is non-uniform so spacing along x changes wrt scaling...
            Vector2f tmp = new Vector2f();
            for ( int yi = 0; yi < nTexSize; ++yi ) {
                for ( int xi = 0; xi < nTexSize; ++xi ) {
                    tmp.X = ((float)xi / (float)nTexSize - 0.5f) * 2.0f;
                    tmp.Y = ((float)yi / (float)nTexSize - 0.5f) * 2.0f;
                    tmp.X *= (fRatio);
                    if ( fRatio != 1 )      // UGLY UGLY HACK!
                        tmp.X *= 1.1f;      
                    float fDist = shadowBox.Distance(tmp);
                    pix.fA = MathUtil.WyvillFalloff(fDist, 0, (1-fInnerRad));
                    m_discShadow.SetPixel(xi, yi, pix);
                }
            }

            // default to fade in-out behavior...
            this.Behaviors.Add(new FadeInOutBehavior(this));
        }


        public string Text {
            get { return m_text; }
            set { m_text = value; }
        }

        public float DropShadowWidth {
            get { return m_fDropShadowWidth; }
            set { m_fDropShadowWidth = value; }
        }


        //override protected void UpdateGeometry() {
        //    m_box = Bounds;
        //}

//        override protected void RenderWidgetBackground() {
        override public void RenderBackground() {
            // drop shadow
            m_box = Bounds;
            Box2f shadowBounds = new Box2f(m_box.Center, 
                m_box.Width/2.0f + m_fDropShadowWidth,
                m_box.Height/2.0f + m_fDropShadowWidth);
            shadowBounds.Translate( new Vector2f(m_box.Height/10,-m_box.Height/10) );
            RenderEngine.Get.DrawImage( m_discShadow, shadowBounds );
        }

//        override protected void RenderWidget() {
        override public void Render() {
            m_box = Bounds;
            ColorRGBA cInterior = ColorRGBA.WHITE;
            cInterior.A *= Opacity;
            ColorRGBA cBorder = ColorRGBA.BLACK;
            cInterior.A *= Opacity;

            RenderEngine.Get.DrawBox(m_box, cBorder);
//            m_box.Expand(-15f);
            m_box.Expand(-1f);
            RenderEngine.Get.DrawBox(m_box, cInterior);

            Vector2f vText = new Vector2f(m_box.Min);
            vText.X += m_box.Width * 0.1f;
            vText.Y += m_box.Width * 0.1f;
            RenderEngine.Get.DrawText(m_text, vText, ColorRGBA.BLACK);
        }

    }
}
