using System;
using System.Collections.Generic;
using System.Text;

namespace Sketcher.CrossKeys
{
    class FingerKeyboard : InfiniteRectContainer
    {
        List< List< char > > m_keyset;
        List<FingerKeyButton> m_keys;

        public FingerKeyboard(Box2f bounds)  : base(bounds, new VerticalAlignedRowLayout()) 
        {
            ((VerticalAlignedRowLayout)base.Layout).Align = VerticalAlignedRowLayout.AlignMode.Center;
            m_keys = new List<FingerKeyButton>();
            SetKeys_Qwerty();
            InitializeKeys();
        }


        void SetKeys_Qwerty() {
            char[] numbers = {'1', '2', '3', '4', '5', '6', '7', '8', '9', '0', '\b'};
            char[] row1 = {'q','w','e','r','t','y','u','i','o','p'};
            char[] row2 = {'a','s','d','f','g','h','j','k','l','\n'};
            char[] row3 = {'z','x','c','v','b','n','m',',','.'};
            char[] row4 = {' '};

            m_keyset = new List<List<char>>();
            m_keyset.Add( new List<char>(numbers));
            m_keyset.Add( new List<char>(row1));
            m_keyset.Add( new List<char>(row2));
            m_keyset.Add( new List<char>(row3));
            m_keyset.Add( new List<char>(row4));
        }

        void InitializeKeys() {

            // count max key width. Keys are w=1 except enter (w=2) and space (w=5)
            int nMaxWidth = 0;
            for ( int i = 0; i < m_keyset.Count; ++i ) {
                int nRowWidth = 0;
                for ( int j = 0; j < m_keyset[i].Count; ++j ) {
                    if ( m_keyset[i][j] == ' ')          nRowWidth += 5;
                    else if ( m_keyset[i][j] == '\n' )   nRowWidth += 2;
                    else if ( m_keyset[i][j] == '\b' )   nRowWidth += 2;
                    else                                 nRowWidth += 1;
                }
                nMaxWidth = (nRowWidth > nMaxWidth) ? nRowWidth : nMaxWidth;
            }

            float fMaxWidth = ChildAreaBounds.Width * 0.95f;
            float fKeySize = fMaxWidth / (int)(nMaxWidth);

            // add top spacer
            Spacer topSpacer = new Spacer(fKeySize/2, fKeySize/2);
            this.Items.Add( topSpacer );
            Layout.AppendHint( new VerticalAlignedRowLayout.NewlineHint(topSpacer) );

            for ( int i = 0; i < m_keyset.Count; ++i ) {
                for ( int j = 0; j < m_keyset[i].Count; ++j ) {
                    float fWidth = fKeySize;
                    if ( m_keyset[i][j] == ' ')          fWidth *= 5.0f;
                    else if ( m_keyset[i][j] == '\n' )   fWidth *= 2.0f;
                    else if ( m_keyset[i][j] == '\b' )   fWidth *= 2.0f;

                    string str = ""+m_keyset[i][j];
                    if ( m_keyset[i][j] == '\n' )
                        str = "Enter";
                    else if ( m_keyset[i][j] == '\b' )
                        str = "Bkspc";
                    FingerKeyButton button = new FingerKeyButton(fWidth, fKeySize, str);

                    this.Items.Add(button);   
                    if ( j == m_keyset[i].Count-1)
                        Layout.AppendHint( new VerticalAlignedRowLayout.NewlineHint(button) );
                }
            }


        }
    }
}
