using System;
using System.Runtime.InteropServices;
using System.ComponentModel;
using System.Threading;

namespace Sketcher
{
    internal class SysTimer
    {
        [DllImport("Kernel32.dll")]
        private static extern bool QueryPerformanceCounter(out long lpPerformanceCount);
        [DllImport("Kernel32.dll")]
        private static extern bool QueryPerformanceFrequency(out long lpFrequency);

        bool m_bRunning;
        private long startTime, stopTime;
        private long freq;

        // Constructor
        public SysTimer() {
            m_bRunning = false;
            startTime = 0;
            stopTime = 0;

            if (QueryPerformanceFrequency(out freq) == false) {
                // high-performance counter not supported
                throw new Win32Exception();
            }
        }

        public bool Running {
            get { return m_bRunning; }
        }

        // Start the timer
        public void Start() {
            QueryPerformanceCounter(out startTime);
            m_bRunning = true;
        }

        // Stop the timer
        public void Stop() {
            QueryPerformanceCounter(out stopTime);
            m_bRunning = false;
        }

        public double Current {
            get {
                long curTime; QueryPerformanceCounter(out curTime);
                return (double)(curTime - startTime) / (double)freq;
            }
        }
        
        // Returns the duration of the timer (in seconds)
        public double Duration {
            get {
                return (double)(stopTime - startTime) / (double)freq;
            }
        }
    }
}