using System;
using System.Collections.Generic;
using System.Text;

namespace Sketcher
{
    class Animation
    {
        protected Renderable m_Target;
        protected float m_fTimeElapsed;
        protected float m_fDuration;
        protected bool m_bPlaying;

        public Animation(Renderable target) {
            m_Target = target;
            m_fTimeElapsed = 0;
            m_fDuration = 0;
            m_bPlaying = false;
        }

        public Animation(Renderable target, float fDuration) {
            m_Target = target;
            m_fTimeElapsed = 0;
            m_fDuration = fDuration;
            m_bPlaying = false;
        }

        public Renderable Target {
            get { return m_Target; }
        }

        public bool Playing {
            get { return m_bPlaying; }
        }
        public float Duration {
            get { return m_fDuration; }
        }
        public float TimeElapsed {
            get { return m_fTimeElapsed; }
        }
        public float TimeElapsedFraction {
            get { return m_fTimeElapsed / m_fDuration; }
        }



        // notification handler for when animation finishes
        public delegate void AnimationFinishedHandler(object o);
        public event AnimationFinishedHandler AnimationFinishedEvent;

        protected void NotifyAnimationFinished() {
            if (AnimationFinishedEvent != null)
                AnimationFinishedEvent(this);
        } 


        /*
         * Animation control. Default animation control is just to uniformly
         * increment time, and call OnTimeUpdated(). Client can just override
         * OnTimeUpdated() to use this simple behavior. Or, override Start/Stop/TimeDelta
         * to implement more advanced animations
         */

        virtual public void Start() {
            m_fTimeElapsed = 0;
            m_bPlaying = true;
            OnTimeUpdated();
        }

        virtual public void Stop() {
            m_fTimeElapsed = m_fDuration;
            OnTimeUpdated();
            m_bPlaying = false;

            // notify listeners that animation has terminated
            NotifyAnimationFinished();
        }

        //! return true if animation is finished
        virtual public bool TimeDelta( float fDelta ) {
            if ( ! m_bPlaying )
                return true;

            m_fTimeElapsed += fDelta;
            if ( m_fTimeElapsed > m_fDuration ) {
                Stop();
                return true;
            };
            OnTimeUpdated();
            return false;
        }


        //! animation action
        virtual protected void OnTimeUpdated() {
            // do something here...
        }

    }
}
