using System;
using System.Collections.Generic;
using System.Text;

namespace Sketcher
{
    class VectorItemTranslater : ItemInputBehavior
    {
        protected Vector2f m_positionVec;
        protected VectorGraphicsItem m_vectorItem;

        public VectorItemTranslater(VectorGraphicsItem target)
            : base(target) {
            m_positionVec = Vector2f.ZERO;
            m_vectorItem = target;
        }

        public VectorGraphicsItem VTarget {
            get { return m_vectorItem; }
        }

        override public InputCaptureType CaptureType {
            get { return InputCaptureType.FirstVertexOrNeverCapture; }
        }

        //! return value in [0,1] to indicate desire for capturing stroke
        override public float GetCaptureScore(Stroke s) {
            return VTarget.IsHit(s.First);
        }

        //! return true to continue capture
        override public bool OnCaptureStrokeStart(Stroke s) {
            VectorCanvas canvas = (VectorCanvas)VTarget.Parent;
            canvas.SelectedItems.SelectItem(VTarget, false);

            m_positionVec = VTarget.Bounds.Min - s[0];
            return true;
        }

        //! return true to continue capture, false to stop capture
        override public bool OnCapturedStrokeUpdate(Stroke s) {
            if (s.Vertices.Count > 1) {
                Box2f bounds = VTarget.Bounds;
                bounds.MoveMin(s.Last + m_positionVec);
                VTarget.ParentContainer.ResizeChild(VTarget, bounds);
            }
            return true;
        }

        override public bool OnCaptureStrokeEnd(Stroke s) {
            VTarget.NotifySizeChanged();
            return true;    // consumed
        }


        //! return click "score"
        override public float GetClickScore(ClickStroke s) {
            if (VTarget.Bounds.Contains(s.Click))
                return 1.0f;
            return 0.0f;
        }

        //! handle click event
        override public void OnClick(ClickStroke s) {
            VectorCanvas canvas = (VectorCanvas)VTarget.Parent;
            canvas.SelectedItems.SelectItem(VTarget, false);
        }

    }
}
