using System;
using System.Collections.Generic;
using System.Text;

namespace Sketcher
{
    class ToolbarWidget : Widget
    {
        Item.ItemBoundsChangedHandler m_TargetResizedHandler;
        FixedRectContainer m_container;
        WidgetContainerBridge m_bridge;


        public enum Location {
            Free,
            TopLeft,
            TopRight,
            BottomLeft,
            BottomRight,
            Centered
        };
        Location m_dock;

        int m_nBorderWidth;
 

        public ToolbarWidget(Element parent, float fWidth, float fHeight, Item dockTarget)
            : base( parent )
        {
            Resize(new Box2f(fWidth, fHeight));

            BackgroundColor = ColorRGBA.WHITE;
            ForegroundColor = ColorRGBA.BLACK;

            m_dock = Location.Free;
            m_nBorderWidth = 1;

            m_TargetResizedHandler = new Item.ItemBoundsChangedHandler( OnTargetResized );
            dockTarget.ItemBoundsChangedEvent += m_TargetResizedHandler;

            m_container = new FixedRectContainer(Bounds, new VerticalTilingLayout());
            m_container.ChildAreaBorder = m_nBorderWidth;
            m_container.ForegroundColor = m_container.BackgroundColor = ColorRGBA.WHITE;

            // have to use bridge to connect container to widget
            m_bridge = new WidgetContainerBridge(this);
            m_bridge.Items.Add(m_container);
            m_container.Parent = m_bridge;

            m_bridge.ElementChangedEvent += new ElementChangedHandler(OnBridgeElementChanged);
            m_bridge.ElementVisibilityChangedEvent += new ElementVisibilityChangedHandler(OnBridgeVisibilityChanged);
            m_bridge.ItemBoundsChangedEvent += new Item.ItemBoundsChangedHandler(OnBridgeBoundsChanged);
        }

        public FixedRectContainer ItemContainer {
            get { return m_container; }
        }

        public Location DockLocation {
            get { return m_dock; }
            set { m_dock = value; OnTargetResized( null );  }
        }

        public int BorderWidth {
            get { return m_nBorderWidth; }
            set { m_nBorderWidth = value; m_container.ChildAreaBorder = m_nBorderWidth; }
        }

        virtual protected void OnTargetResized( object o ) {
            if (Parent == null)
                return;

            if (m_dock != Location.Free) {
                Box2f curBox = Bounds;
                Box2f canvasBox = Parent.Bounds;
                Vector2f vTranslate = Vector2f.ZERO;
                switch (m_dock) {
                    case Location.Centered:
                        vTranslate = canvasBox.Center - curBox.Center;
                        break;
                    case Location.BottomLeft:
                        vTranslate = canvasBox.GetCorner( 0 ) - curBox.GetCorner( 0 );
                        break;
                    case Location.BottomRight:
                        vTranslate = canvasBox.GetCorner( 1 ) - curBox.GetCorner( 1 );
                        break;
                    case Location.TopRight:
                        vTranslate = canvasBox.GetCorner( 2 ) - curBox.GetCorner( 2 );
                        break;
                    case Location.TopLeft:
                        vTranslate = canvasBox.GetCorner( 3 ) - curBox.GetCorner( 3 );
                        break;
                }
                curBox.Translate( vTranslate );
                Resize( curBox );
            }

            m_bridge.Resize(Bounds);
            //m_container.Resize(Bounds);
        }

        /*
         * Widget Overrides
         */

        public override void Disconnect( ) {
            base.Disconnect();
            m_bridge.Disconnect();
        }

        override protected void RenderWidget() {
            m_bridge.Render();
            if ( m_nBorderWidth > 0 )
                RenderEngine.Get.DrawBoxOutline( Bounds, AlphaForegroundColor );
        }



        /*
         * need to listen to bridge events
         */
        void OnBridgeElementChanged(object o) {
            NotifyElementChanged();
        }
        void OnBridgeVisibilityChanged(object o) {
            NotifyElementVisibilityChanged();
        }
        void OnBridgeBoundsChanged(object o) {
            NotifyElementChanged();
        }
    }
}
